/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSCalendar;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSDate;
import dwt.internal.cocoa.NSDateComponents;
import dwt.internal.cocoa.NSLocale;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSRange;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSTimeZone;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSCalendar : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public static id autoupdatingCurrentCalendar ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSCalendar, OS.sel_autoupdatingCurrentCalendar);
        return result !is null ? new id(result) : null;
    }

    public NSString calendarIdentifier ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_calendarIdentifier);
        return result !is null ? new NSString(result) : null;
    }

    public NSDateComponents components_fromDate_ (NSUInteger unitFlags, NSDate date)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_components_1fromDate_1, unitFlags, date !is null ? date.id : null);
        return result !is null ? new NSDateComponents(result) : null;
    }

    public NSDateComponents components_fromDate_toDate_options_ (NSUInteger unitFlags, NSDate startingDate, NSDate resultDate, NSUInteger opts)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_components_1fromDate_1toDate_1options_1, unitFlags,
                startingDate !is null ? startingDate.id : null, resultDate !is null ? resultDate.id : null, opts);
        return result !is null ? new NSDateComponents(result) : null;
    }

    public static id currentCalendar ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSCalendar, OS.sel_currentCalendar);
        return result !is null ? new id(result) : null;
    }

    public NSDate dateByAddingComponents (NSDateComponents comps, NSDate date, NSUInteger opts)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_dateByAddingComponents_1toDate_1options_1, comps !is null ? comps.id : null,
                date !is null ? date.id : null, opts);
        return result !is null ? new NSDate(result) : null;
    }

    public NSDate dateFromComponents (NSDateComponents comps)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_dateFromComponents_1, comps !is null ? comps.id : null);
        return result !is null ? new NSDate(result) : null;
    }

    public NSUInteger firstWeekday ()
    {
        return OS.objc_msgSend(this.id, OS.sel_firstWeekday);
    }

    public NSCalendar initWithCalendarIdentifier (NSString ident)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithCalendarIdentifier_1, ident !is null ? ident.id : null);
        return result !is null ? this : null;
    }

    public NSLocale locale ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_locale);
        return result !is null ? new NSLocale(result) : null;
    }

    public NSRange maximumRangeOfUnit (objc.id unit)
    {
        NSRange result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_maximumRangeOfUnit_1, unit);
        return result;
    }

    public NSUInteger minimumDaysInFirstWeek ()
    {
        return OS.objc_msgSend(this.id, OS.sel_minimumDaysInFirstWeek);
    }

    public NSRange minimumRangeOfUnit (objc.id unit)
    {
        NSRange result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_minimumRangeOfUnit_1, unit);
        return result;
    }

    public NSUInteger ordinalityOfUnit (objc.id smaller, objc.id larger, NSDate date)
    {
        return OS.objc_msgSend(this.id, OS.sel_ordinalityOfUnit_1inUnit_1forDate_1, smaller, larger, date !is null ? date.id : null);
    }

    public NSRange rangeOfUnit_inUnit_forDate_ (objc.id smaller, objc.id larger, NSDate date)
    {
        NSRange result;
        OS.objc_msgSend_stret(result, this.id, OS.sel_rangeOfUnit_1inUnit_1forDate_1, smaller, larger, date !is null ? date.id : null);
        return result;
    }

    public bool rangeOfUnit_startDate_interval_forDate_ (objc.id unit, objc.id datep, objc.id tip, NSDate date)
    {
        return OS.objc_msgSend(this.id, OS.sel_rangeOfUnit_1startDate_1interval_1forDate_1, unit, datep, tip, date !is null ? date.id : null) !is null;
    }

    public void setFirstWeekday (NSUInteger weekday)
    {
        OS.objc_msgSend(this.id, OS.sel_setFirstWeekday_1, weekday);
    }

    public void setLocale (NSLocale locale)
    {
        OS.objc_msgSend(this.id, OS.sel_setLocale_1, locale !is null ? locale.id : null);
    }

    public void setMinimumDaysInFirstWeek (NSUInteger mdw)
    {
        OS.objc_msgSend(this.id, OS.sel_setMinimumDaysInFirstWeek_1, mdw);
    }

    public void setTimeZone (NSTimeZone tz)
    {
        OS.objc_msgSend(this.id, OS.sel_setTimeZone_1, tz !is null ? tz.id : null);
    }

    public NSTimeZone timeZone ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_timeZone);
        return result !is null ? new NSTimeZone(result) : null;
    }

}
