/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSError;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSError : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSInteger code ()
    {
        return OS.objc_msgSend(this.id, OS.sel_code);
    }

    public NSString domain ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_domain);
        return result !is null ? new NSString(result) : null;
    }

    public static id errorWithDomain (NSString domain, NSInteger code, NSDictionary dict)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSError, OS.sel_errorWithDomain_1code_1userInfo_1, domain !is null ? domain.id : null, code,
                dict !is null ? dict.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithDomain (NSString domain, NSInteger code, NSDictionary dict)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithDomain_1code_1userInfo_1, domain !is null ? domain.id : null, code,
                dict !is null ? dict.id : null);
        return result !is null ? new id(result) : null;
    }

    public NSString localizedDescription ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_localizedDescription);
        return result !is null ? new NSString(result) : null;
    }

    public NSString localizedFailureReason ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_localizedFailureReason);
        return result !is null ? new NSString(result) : null;
    }

    public NSArray localizedRecoveryOptions ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_localizedRecoveryOptions);
        return result !is null ? new NSArray(result) : null;
    }

    public NSString localizedRecoverySuggestion ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_localizedRecoverySuggestion);
        return result !is null ? new NSString(result) : null;
    }

    public id recoveryAttempter ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_recoveryAttempter);
        return result !is null ? new id(result) : null;
    }

    public NSDictionary userInfo ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_userInfo);
        return result !is null ? new NSDictionary(result) : null;
    }

}
