/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSFileWrapper;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSFileWrapper;
import dwt.internal.cocoa.NSImage;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSFileWrapper : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSString addFileWithPath (NSString path)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_addFileWithPath_1, path !is null ? path.id : null);
        return result !is null ? new NSString(result) : null;
    }

    public NSString addFileWrapper (NSFileWrapper doc)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_addFileWrapper_1, doc !is null ? doc.id : null);
        return result !is null ? new NSString(result) : null;
    }

    public NSString addRegularFileWithContents (NSData data, NSString filename)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_addRegularFileWithContents_1preferredFilename_1, data !is null ? data.id : null,
                filename !is null ? filename.id : null);
        return result !is null ? new NSString(result) : null;
    }

    public NSString addSymbolicLinkWithDestination (NSString path, NSString filename)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_addSymbolicLinkWithDestination_1preferredFilename_1, path !is null ? path.id : null,
                filename !is null ? filename.id : null);
        return result !is null ? new NSString(result) : null;
    }

    public NSDictionary fileAttributes ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_fileAttributes);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSDictionary fileWrappers ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_fileWrappers);
        return result !is null ? new NSDictionary(result) : null;
    }

    public NSString filename ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_filename);
        return result !is null ? new NSString(result) : null;
    }

    public NSImage icon ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_icon);
        return result !is null ? new NSImage(result) : null;
    }

    public id initDirectoryWithFileWrappers (NSDictionary docs)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initDirectoryWithFileWrappers_1, docs !is null ? docs.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initRegularFileWithContents (NSData data)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initRegularFileWithContents_1, data !is null ? data.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initSymbolicLinkWithDestination (NSString path)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initSymbolicLinkWithDestination_1, path !is null ? path.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithPath (NSString path)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithPath_1, path !is null ? path.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithSerializedRepresentation (NSData data)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithSerializedRepresentation_1, data !is null ? data.id : null);
        return result !is null ? new id(result) : null;
    }

    public bool isDirectory ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isDirectory) !is null;
    }

    public bool isRegularFile ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isRegularFile) !is null;
    }

    public bool isSymbolicLink ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isSymbolicLink) !is null;
    }

    public NSString keyForFileWrapper (NSFileWrapper doc)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_keyForFileWrapper_1, doc !is null ? doc.id : null);
        return result !is null ? new NSString(result) : null;
    }

    public bool needsToBeUpdatedFromPath (NSString path)
    {
        return OS.objc_msgSend(this.id, OS.sel_needsToBeUpdatedFromPath_1, path !is null ? path.id : null) !is null;
    }

    public NSString preferredFilename ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_preferredFilename);
        return result !is null ? new NSString(result) : null;
    }

    public NSData regularFileContents ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_regularFileContents);
        return result !is null ? new NSData(result) : null;
    }

    public void removeFileWrapper (NSFileWrapper doc)
    {
        OS.objc_msgSend(this.id, OS.sel_removeFileWrapper_1, doc !is null ? doc.id : null);
    }

    public NSData serializedRepresentation ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_serializedRepresentation);
        return result !is null ? new NSData(result) : null;
    }

    public void setFileAttributes (NSDictionary attributes)
    {
        OS.objc_msgSend(this.id, OS.sel_setFileAttributes_1, attributes !is null ? attributes.id : null);
    }

    public void setFilename (NSString filename)
    {
        OS.objc_msgSend(this.id, OS.sel_setFilename_1, filename !is null ? filename.id : null);
    }

    public void setIcon (NSImage icon)
    {
        OS.objc_msgSend(this.id, OS.sel_setIcon_1, icon !is null ? icon.id : null);
    }

    public void setPreferredFilename (NSString filename)
    {
        OS.objc_msgSend(this.id, OS.sel_setPreferredFilename_1, filename !is null ? filename.id : null);
    }

    public NSString symbolicLinkDestination ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_symbolicLinkDestination);
        return result !is null ? new NSString(result) : null;
    }

    public bool updateFromPath (NSString path)
    {
        return OS.objc_msgSend(this.id, OS.sel_updateFromPath_1, path !is null ? path.id : null) !is null;
    }

    public bool writeToFile (NSString path, bool atomicFlag, bool updateFilenamesFlag)
    {
        return OS.objc_msgSend(this.id, OS.sel_writeToFile_1atomically_1updateFilenames_1, path !is null ? path.id : null, atomicFlag,
                updateFilenamesFlag) !is null;
    }

}
