/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSHashTable;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSEnumerator;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSPointerFunctions;
import dwt.internal.cocoa.NSSet;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSHashTable : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void addObject (id object)
    {
        OS.objc_msgSend(this.id, OS.sel_addObject_1, object !is null ? object.id : null);
    }

    public NSArray allObjects ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_allObjects);
        return result !is null ? new NSArray(result) : null;
    }

    public id anyObject ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_anyObject);
        return result !is null ? new id(result) : null;
    }

    public bool containsObject (id anObject)
    {
        return OS.objc_msgSend(this.id, OS.sel_containsObject_1, anObject !is null ? anObject.id : null) !is null;
    }

    public NSUInteger count ()
    {
        return OS.objc_msgSend(this.id, OS.sel_count);
    }

    public static id hashTableWithOptions (NSPointerFunctionsOptions options)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSHashTable, OS.sel_hashTableWithOptions_1, options);
        return result !is null ? new id(result) : null;
    }

    public static id hashTableWithWeakObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSHashTable, OS.sel_hashTableWithWeakObjects);
        return result !is null ? new id(result) : null;
    }

    public id initWithOptions (NSPointerFunctionsOptions options, NSUInteger initialCapacity)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithOptions_1capacity_1, options, initialCapacity);
        return result !is null ? new id(result) : null;
    }

    public id initWithPointerFunctions (NSPointerFunctions functions, NSUInteger initialCapacity)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithPointerFunctions_1capacity_1, functions !is null ? functions.id : null,
                initialCapacity);
        return result !is null ? new id(result) : null;
    }

    public void intersectHashTable (NSHashTable other)
    {
        OS.objc_msgSend(this.id, OS.sel_intersectHashTable_1, other !is null ? other.id : null);
    }

    public bool intersectsHashTable (NSHashTable other)
    {
        return OS.objc_msgSend(this.id, OS.sel_intersectsHashTable_1, other !is null ? other.id : null) !is null;
    }

    public bool isEqualToHashTable (NSHashTable other)
    {
        return OS.objc_msgSend(this.id, OS.sel_isEqualToHashTable_1, other !is null ? other.id : null) !is null;
    }

    public bool isSubsetOfHashTable (NSHashTable other)
    {
        return OS.objc_msgSend(this.id, OS.sel_isSubsetOfHashTable_1, other !is null ? other.id : null) !is null;
    }

    public id member (id object)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_member_1, object !is null ? object.id : null);
        return result !is null ? new id(result) : null;
    }

    public void minusHashTable (NSHashTable other)
    {
        OS.objc_msgSend(this.id, OS.sel_minusHashTable_1, other !is null ? other.id : null);
    }

    public NSEnumerator objectEnumerator ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_objectEnumerator);
        return result !is null ? new NSEnumerator(result) : null;
    }

    public NSPointerFunctions pointerFunctions ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_pointerFunctions);
        return result !is null ? new NSPointerFunctions(result) : null;
    }

    public void removeAllObjects ()
    {
        OS.objc_msgSend(this.id, OS.sel_removeAllObjects);
    }

    public void removeObject (id object)
    {
        OS.objc_msgSend(this.id, OS.sel_removeObject_1, object !is null ? object.id : null);
    }

    public NSSet setRepresentation ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_setRepresentation);
        return result !is null ? new NSSet(result) : null;
    }

    public void unionHashTable (NSHashTable other)
    {
        OS.objc_msgSend(this.id, OS.sel_unionHashTable_1, other !is null ? other.id : null);
    }

}
