/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSMachPort;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSPort;
import dwt.internal.cocoa.NSRunLoop;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSMachPort : NSPort
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public id initWithMachPort_ (uint machPort)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithMachPort_1, machPort);
        return result !is null ? new id(result) : null;
    }

    public id initWithMachPort_options_ (uint machPort, NSUInteger f)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithMachPort_1options_1, machPort, f);
        return result !is null ? new id(result) : null;
    }

    public uint machPort ()
    {
        return OS.objc_msgSend(this.id, OS.sel_machPort);
    }

    public static NSPort static_portWithMachPort_ (uint machPort)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMachPort, OS.sel_portWithMachPort_1, machPort);
        return result !is null ? new NSPort(result) : null;
    }

    public static NSPort static_portWithMachPort_options_ (uint machPort, NSUInteger f)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSMachPort, OS.sel_portWithMachPort_1options_1, machPort, f);
        return result !is null ? new NSPort(result) : null;
    }

    public void removeFromRunLoop (NSRunLoop runLoop, NSString mode)
    {
        OS.objc_msgSend(this.id, OS.sel_removeFromRunLoop_1forMode_1, runLoop !is null ? runLoop.id : null, mode !is null ? mode.id : null);
    }

    public void scheduleInRunLoop (NSRunLoop runLoop, NSString mode)
    {
        OS.objc_msgSend(this.id, OS.sel_scheduleInRunLoop_1forMode_1, runLoop !is null ? runLoop.id : null, mode !is null ? mode.id : null);
    }

}
