/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSNotificationCenter;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSNotification;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSNotificationCenter : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void addObserver (id observer, objc.SEL aSelector, NSString aName, id anObject)
    {
        OS.objc_msgSend(this.id, OS.sel_addObserver_1selector_1name_1object_1, observer !is null ? observer.id : null, aSelector,
                aName !is null ? aName.id : null, anObject !is null ? anObject.id : null);
    }

    public static NSNotificationCenter defaultCenter ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSNotificationCenter, OS.sel_defaultCenter);
        return result !is null ? new NSNotificationCenter(result) : null;
    }

    public void postNotification (NSNotification notification)
    {
        OS.objc_msgSend(this.id, OS.sel_postNotification_1, notification !is null ? notification.id : null);
    }

    public void postNotificationName_object_ (NSString aName, id anObject)
    {
        OS.objc_msgSend(this.id, OS.sel_postNotificationName_1object_1, aName !is null ? aName.id : null, anObject !is null ? anObject.id : null);
    }

    public void postNotificationName_object_userInfo_ (NSString aName, id anObject, NSDictionary aUserInfo)
    {
        OS.objc_msgSend(this.id, OS.sel_postNotificationName_1object_1userInfo_1, aName !is null ? aName.id : null,
                anObject !is null ? anObject.id : null, aUserInfo !is null ? aUserInfo.id : null);
    }

    public void removeObserver_ (id observer)
    {
        OS.objc_msgSend(this.id, OS.sel_removeObserver_1, observer !is null ? observer.id : null);
    }

    public void removeObserver_name_object_ (id observer, NSString aName, id anObject)
    {
        OS.objc_msgSend(this.id, OS.sel_removeObserver_1name_1object_1, observer !is null ? observer.id : null, aName !is null ? aName.id : null,
                anObject !is null ? anObject.id : null);
    }

}
