/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSOpenGLView;

import dwt.internal.cocoa.NSOpenGLContext;
import dwt.internal.cocoa.NSOpenGLPixelFormat;
import dwt.internal.cocoa.NSRect;
import dwt.internal.cocoa.NSView;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSOpenGLView : NSView
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void clearGLContext ()
    {
        OS.objc_msgSend(this.id, OS.sel_clearGLContext);
    }

    public static NSOpenGLPixelFormat defaultPixelFormat ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSOpenGLView, OS.sel_defaultPixelFormat);
        return result !is null ? new NSOpenGLPixelFormat(result) : null;
    }

    public NSOpenGLView initWithFrame (NSRect frameRect, NSOpenGLPixelFormat format)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithFrame_1pixelFormat_1, frameRect, format !is null ? format.id : null);
        return result !is null ? this : null;
    }

    public NSOpenGLContext openGLContext ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_openGLContext);
        return result !is null ? new NSOpenGLContext(result) : null;
    }

    public NSOpenGLPixelFormat pixelFormat ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_pixelFormat);
        return result !is null ? new NSOpenGLPixelFormat(result) : null;
    }

    public void prepareOpenGL ()
    {
        OS.objc_msgSend(this.id, OS.sel_prepareOpenGL);
    }

    public void reshape ()
    {
        OS.objc_msgSend(this.id, OS.sel_reshape);
    }

    public void setOpenGLContext (NSOpenGLContext context)
    {
        OS.objc_msgSend(this.id, OS.sel_setOpenGLContext_1, context !is null ? context.id : null);
    }

    public void setPixelFormat (NSOpenGLPixelFormat pixelFormat)
    {
        OS.objc_msgSend(this.id, OS.sel_setPixelFormat_1, pixelFormat !is null ? pixelFormat.id : null);
    }

    public void update ()
    {
        OS.objc_msgSend(this.id, OS.sel_update);
    }

}
