/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSOutputStream;

import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSStream;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSOutputStream : NSStream
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public bool hasSpaceAvailable ()
    {
        return OS.objc_msgSend(this.id, OS.sel_hasSpaceAvailable) !is null;
    }

    public NSOutputStream initToBuffer (ubyte* buffer, NSUInteger capacity)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initToBuffer_1capacity_1, buffer, capacity);
        return result !is null ? this : null;
    }

    public NSOutputStream initToFileAtPath (NSString path, bool shouldAppend)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initToFileAtPath_1append_1, path !is null ? path.id : null, shouldAppend);
        return result !is null ? this : null;
    }

    public NSOutputStream initToMemory ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initToMemory);
        return result !is null ? this : null;
    }

    public static id outputStreamToBuffer (ubyte* buffer, NSUInteger capacity)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSOutputStream, OS.sel_outputStreamToBuffer_1capacity_1, buffer, capacity);
        return result !is null ? new id(result) : null;
    }

    public static id outputStreamToFileAtPath (NSString path, bool shouldAppend)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSOutputStream, OS.sel_outputStreamToFileAtPath_1append_1, path !is null ? path.id : null,
                shouldAppend);
        return result !is null ? new id(result) : null;
    }

    public static id outputStreamToMemory ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSOutputStream, OS.sel_outputStreamToMemory);
        return result !is null ? new id(result) : null;
    }

    public NSInteger write (/*const*/ubyte* buffer, NSUInteger len)
    {
        return cast(NSInteger) OS.objc_msgSend(this.id, OS.sel_write_1maxLength_1, buffer, len);
    }

}
