/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSPointerArray;

import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSPointerFunctions;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSPointerArray : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void addPointer (void* pointer)
    {
        OS.objc_msgSend(this.id, OS.sel_addPointer_1, pointer);
    }

    public NSArray allObjects ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_allObjects);
        return result !is null ? new NSArray(result) : null;
    }

    public void compact ()
    {
        OS.objc_msgSend(this.id, OS.sel_compact);
    }

    public NSUInteger count ()
    {
        return OS.objc_msgSend(this.id, OS.sel_count);
    }

    public NSPointerArray initWithOptions (NSPointerFunctionsOptions options)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithOptions_1, options);
        return result !is null ? this : null;
    }

    public id initWithPointerFunctions (NSPointerFunctions functions)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithPointerFunctions_1, functions !is null ? functions.id : null);
        return result !is null ? new id(result) : null;
    }

    public void insertPointer (void* item, NSUInteger index)
    {
        OS.objc_msgSend(this.id, OS.sel_insertPointer_1atIndex_1, item, index);
    }

    public static id pointerArrayWithOptions (NSPointerFunctionsOptions options)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSPointerArray, OS.sel_pointerArrayWithOptions_1, options);
        return result !is null ? new id(result) : null;
    }

    public static id pointerArrayWithPointerFunctions (NSPointerFunctions functions)
    {
        objc.id
                result = OS.objc_msgSend(OS.class_NSPointerArray, OS.sel_pointerArrayWithPointerFunctions_1, functions !is null ? functions.id : null);
        return result !is null ? new id(result) : null;
    }

    public static id pointerArrayWithStrongObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSPointerArray, OS.sel_pointerArrayWithStrongObjects);
        return result !is null ? new id(result) : null;
    }

    public static id pointerArrayWithWeakObjects ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSPointerArray, OS.sel_pointerArrayWithWeakObjects);
        return result !is null ? new id(result) : null;
    }

    public void* pointerAtIndex (NSUInteger index)
    {
        return cast(void*) OS.objc_msgSend(this.id, OS.sel_pointerAtIndex_1, index);
    }

    public NSPointerFunctions pointerFunctions ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_pointerFunctions);
        return result !is null ? new NSPointerFunctions(result) : null;
    }

    public void removePointerAtIndex (NSUInteger index)
    {
        OS.objc_msgSend(this.id, OS.sel_removePointerAtIndex_1, index);
    }

    public void replacePointerAtIndex (NSUInteger index, void* item)
    {
        OS.objc_msgSend(this.id, OS.sel_replacePointerAtIndex_1withPointer_1, index, item);
    }

    public void setCount (NSUInteger count)
    {
        OS.objc_msgSend(this.id, OS.sel_setCount_1, count);
    }

}
