/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSTask;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSArray;
import dwt.internal.cocoa.NSDictionary;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSTask : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSArray arguments ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_arguments);
        return result !is null ? new NSArray(result) : null;
    }

    public NSString currentDirectoryPath ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_currentDirectoryPath);
        return result !is null ? new NSString(result) : null;
    }

    public NSDictionary environment ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_environment);
        return result !is null ? new NSDictionary(result) : null;
    }

    public void interrupt ()
    {
        OS.objc_msgSend(this.id, OS.sel_interrupt);
    }

    public bool isRunning ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isRunning) !is null;
    }

    public void launch ()
    {
        OS.objc_msgSend(this.id, OS.sel_launch);
    }

    public NSString launchPath ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_launchPath);
        return result !is null ? new NSString(result) : null;
    }

    public static NSTask launchedTaskWithLaunchPath (NSString path, NSArray arguments)
    {
        objc.id result = OS.objc_msgSend(OS.class_NSTask, OS.sel_launchedTaskWithLaunchPath_1arguments_1, path !is null ? path.id : null,
                arguments !is null ? arguments.id : null);
        return result !is null ? new NSTask(result) : null;
    }

    public int processIdentifier ()
    {
        return cast(int) OS.objc_msgSend(this.id, OS.sel_processIdentifier);
    }

    public bool resume ()
    {
        return OS.objc_msgSend(this.id, OS.sel_resume) !is null;
    }

    public void setArguments (NSArray arguments)
    {
        OS.objc_msgSend(this.id, OS.sel_setArguments_1, arguments !is null ? arguments.id : null);
    }

    public void setCurrentDirectoryPath (NSString path)
    {
        OS.objc_msgSend(this.id, OS.sel_setCurrentDirectoryPath_1, path !is null ? path.id : null);
    }

    public void setEnvironment (NSDictionary dict)
    {
        OS.objc_msgSend(this.id, OS.sel_setEnvironment_1, dict !is null ? dict.id : null);
    }

    public void setLaunchPath (NSString path)
    {
        OS.objc_msgSend(this.id, OS.sel_setLaunchPath_1, path !is null ? path.id : null);
    }

    public void setStandardError (id error)
    {
        OS.objc_msgSend(this.id, OS.sel_setStandardError_1, error !is null ? error.id : null);
    }

    public void setStandardInput (id input)
    {
        OS.objc_msgSend(this.id, OS.sel_setStandardInput_1, input !is null ? input.id : null);
    }

    public void setStandardOutput (id output)
    {
        OS.objc_msgSend(this.id, OS.sel_setStandardOutput_1, output !is null ? output.id : null);
    }

    public id standardError ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_standardError);
        return result !is null ? new id(result) : null;
    }

    public id standardInput ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_standardInput);
        return result !is null ? new id(result) : null;
    }

    public id standardOutput ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_standardOutput);
        return result !is null ? new id(result) : null;
    }

    public bool suspend ()
    {
        return OS.objc_msgSend(this.id, OS.sel_suspend) !is null;
    }

    public void terminate ()
    {
        OS.objc_msgSend(this.id, OS.sel_terminate);
    }

    public int terminationStatus ()
    {
        return cast(int) OS.objc_msgSend(this.id, OS.sel_terminationStatus);
    }

    public void waitUntilExit ()
    {
        OS.objc_msgSend(this.id, OS.sel_waitUntilExit);
    }

}
