/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSTokenField;

import dwt.internal.cocoa.NSCharacterSet;
import dwt.internal.cocoa.NSDate : NSTimeInterval;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSTextField;
import dwt.internal.cocoa.NSTokenFieldCell : NSTokenStyle;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSTokenField : NSTextField
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSTimeInterval completionDelay ()
    {
        return cast(NSTimeInterval) OS.objc_msgSend_fpret(this.id, OS.sel_completionDelay);
    }

    public static NSTimeInterval defaultCompletionDelay ()
    {
        return cast(NSTimeInterval) OS.objc_msgSend_fpret(OS.class_NSTokenField, OS.sel_defaultCompletionDelay);
    }

    public static NSCharacterSet defaultTokenizingCharacterSet ()
    {
        objc.id result = OS.objc_msgSend(OS.class_NSTokenField, OS.sel_defaultTokenizingCharacterSet);
        return result !is null ? new NSCharacterSet(result) : null;
    }

    public void setCompletionDelay (NSTimeInterval delay)
    {
        OS.objc_msgSend(this.id, OS.sel_setCompletionDelay_1, delay);
    }

    public void setTokenStyle (NSTokenStyle style)
    {
        OS.objc_msgSend(this.id, OS.sel_setTokenStyle_1, style);
    }

    public void setTokenizingCharacterSet (NSCharacterSet characterSet)
    {
        OS.objc_msgSend(this.id, OS.sel_setTokenizingCharacterSet_1, characterSet !is null ? characterSet.id : null);
    }

    public NSTokenStyle tokenStyle ()
    {
        return cast(NSTokenStyle) OS.objc_msgSend(this.id, OS.sel_tokenStyle);
    }

    public NSCharacterSet tokenizingCharacterSet ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_tokenizingCharacterSet);
        return result !is null ? new NSCharacterSet(result) : null;
    }

}
