/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSURLDownload;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSURLRequest;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSURLDownload : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public static bool canResumeDownloadDecodedWithEncodingMIMEType (NSString MIMEType)
    {
        return OS.objc_msgSend(OS.class_NSURLDownload, OS.sel_canResumeDownloadDecodedWithEncodingMIMEType_1, MIMEType !is null ? MIMEType.id : null) !is null;
    }

    public void cancel ()
    {
        OS.objc_msgSend(this.id, OS.sel_cancel);
    }

    public bool deletesFileUponFailure ()
    {
        return OS.objc_msgSend(this.id, OS.sel_deletesFileUponFailure) !is null;
    }

    public id initWithRequest (NSURLRequest request, id delegatee)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithRequest_1delegate_1, request !is null ? request.id : null,
                delegatee !is null ? delegatee.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithResumeData (NSData resumeData, id delegatee, NSString path)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithResumeData_1delegate_1path_1, resumeData !is null ? resumeData.id : null,
                delegatee !is null ? delegatee.id : null, path !is null ? path.id : null);
        return result !is null ? new id(result) : null;
    }

    public NSURLRequest request ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_request);
        return result !is null ? new NSURLRequest(result) : null;
    }

    public NSData resumeData ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_resumeData);
        return result !is null ? new NSData(result) : null;
    }

    public void setDeletesFileUponFailure (bool deletesFileUponFailure)
    {
        OS.objc_msgSend(this.id, OS.sel_setDeletesFileUponFailure_1, deletesFileUponFailure);
    }

    public void setDestination (NSString path, bool allowOverwrite)
    {
        OS.objc_msgSend(this.id, OS.sel_setDestination_1allowOverwrite_1, path !is null ? path.id : null, allowOverwrite);
    }

}
