/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSURLProtectionSpace;

import dwt.internal.cocoa.id;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSURLProtectionSpace : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public NSString authenticationMethod ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_authenticationMethod);
        return result !is null ? new NSString(result) : null;
    }

    public NSString host ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_host);
        return result !is null ? new NSString(result) : null;
    }

    public id initWithHost (NSString host, NSInteger port, NSString protocol, NSString realm, NSString authenticationMethod)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithHost_1port_1protocol_1realm_1authenticationMethod_1, host !is null ? host.id : null,
                port, protocol !is null ? protocol.id : null, realm !is null ? realm.id : null,
                authenticationMethod !is null ? authenticationMethod.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithProxyHost (NSString host, NSInteger port, NSString type, NSString realm, NSString authenticationMethod)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithProxyHost_1port_1type_1realm_1authenticationMethod_1,
                host !is null ? host.id : null, port, type !is null ? type.id : null, realm !is null ? realm.id : null,
                authenticationMethod !is null ? authenticationMethod.id : null);
        return result !is null ? new id(result) : null;
    }

    public bool isProxy ()
    {
        return OS.objc_msgSend(this.id, OS.sel_isProxy) !is null;
    }

    public NSInteger port ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id, OS.sel_port);
    }

    public NSString protocol ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_protocol);
        return result !is null ? new NSString(result) : null;
    }

    public NSString proxyType ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_proxyType);
        return result !is null ? new NSString(result) : null;
    }

    public NSString realm ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_realm);
        return result !is null ? new NSString(result) : null;
    }

    public bool receivesCredentialSecurely ()
    {
        return OS.objc_msgSend(this.id, OS.sel_receivesCredentialSecurely) !is null;
    }

}
