/*******************************************************************************
 * Copyright (c) 2007 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *     
 * Port to the D Programming language:
 *     Jacob Carlborg <jacob.carlborg@gmail.com>
 *******************************************************************************/
module dwt.internal.cocoa.NSXMLParser;

import dwt.internal.cocoa.NSData;
import dwt.internal.cocoa.NSError;
import dwt.internal.cocoa.NSInteger;
import dwt.internal.cocoa.NSObject;
import dwt.internal.cocoa.NSString;
import dwt.internal.cocoa.NSURL;
import dwt.internal.cocoa.OS;
import objc = dwt.internal.objc.runtime;

public class NSXMLParser : NSObject
{

    public this ()
    {
        super();
    }

    public this (objc.id id)
    {
        super(id);
    }

    public void abortParsing ()
    {
        OS.objc_msgSend(this.id, OS.sel_abortParsing);
    }

    public NSInteger columnNumber ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id, OS.sel_columnNumber);
    }

    public id delegatee ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_delegate);
        return result !is null ? new id(result) : null;
    }

    public id initWithContentsOfURL (NSURL url)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithContentsOfURL_1, url !is null ? url.id : null);
        return result !is null ? new id(result) : null;
    }

    public id initWithData (NSData data)
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_initWithData_1, data !is null ? data.id : null);
        return result !is null ? new id(result) : null;
    }

    public NSInteger lineNumber ()
    {
        return cast(NSInteger) OS.objc_msgSend(this.id, OS.sel_lineNumber);
    }

    public bool parse ()
    {
        return OS.objc_msgSend(this.id, OS.sel_parse) !is null;
    }

    public NSError parserError ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_parserError);
        return result !is null ? new NSError(result) : null;
    }

    public NSString publicID ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_publicID);
        return result !is null ? new NSString(result) : null;
    }

    public void setDelegate (id delegatee)
    {
        OS.objc_msgSend(this.id, OS.sel_setDelegate_1, delegatee !is null ? delegatee.id : null);
    }

    public void setShouldProcessNamespaces (bool shouldProcessNamespaces)
    {
        OS.objc_msgSend(this.id, OS.sel_setShouldProcessNamespaces_1, shouldProcessNamespaces);
    }

    public void setShouldReportNamespacePrefixes (bool shouldReportNamespacePrefixes)
    {
        OS.objc_msgSend(this.id, OS.sel_setShouldReportNamespacePrefixes_1, shouldReportNamespacePrefixes);
    }

    public void setShouldResolveExternalEntities (bool shouldResolveExternalEntities)
    {
        OS.objc_msgSend(this.id, OS.sel_setShouldResolveExternalEntities_1, shouldResolveExternalEntities);
    }

    public bool shouldProcessNamespaces ()
    {
        return OS.objc_msgSend(this.id, OS.sel_shouldProcessNamespaces) !is null;
    }

    public bool shouldReportNamespacePrefixes ()
    {
        return OS.objc_msgSend(this.id, OS.sel_shouldReportNamespacePrefixes) !is null;
    }

    public bool shouldResolveExternalEntities ()
    {
        return OS.objc_msgSend(this.id, OS.sel_shouldResolveExternalEntities) !is null;
    }

    public NSString systemID ()
    {
        objc.id result = OS.objc_msgSend(this.id, OS.sel_systemID);
        return result !is null ? new NSString(result) : null;
    }

}
